import re

TWITTER_USERNAME_PATT = re.compile(r"@(\S+)")
TWITTER_HASHTAG_PATT = re.compile(r"#(\S+)")

# based on https://stackoverflow.com/a/49986645
EMOJI_PATT = re.compile(
    pattern="["
    "\U00002600-\U000026FF"  # misc symbols
    "\U00002700-\U000027BF"  # dingbats
    "\U0001F1E0-\U0001F1FF"  # flags (iOS)
    "\U0001F300-\U0001F5FF"  # symbols & pictographs
    "\U0001F600-\U0001F64F"  # emoticons
    "\U0001F680-\U0001F6FF"  # transport & map symbols
    "\U0001F900-\U0001F9FF"  # Supplemental Symbols and Pictographs
    "]+",
    flags=re.UNICODE,
)

CHAR_PATT = re.compile(r"start_char=([0-9]*)\|end_char=([0-9]*)")

CHAR_REPLACEMENTS = {
    "ö": "oe",
    "Ö": "Oe",
    "ü": "ue",
    "Ü": "Ue",
    "ä": "ae",
    "Ä": "Ae",
    "ß": "ss",
    "ō": "oe",  # encountered in plandok (OCR err?)
    "ó": "o1",
    "È": "E1",
    "é": "e1",
    "è": "e1",
    "á": "a1",
    "ï": "i2",
    "Ã": "A",
    "Â²": "A2",
}

PUNCT_REPLACEMENTS = {
    "+": "PLUS",
    "/": "SLASH",
    "\\": "BACKSLASH",
    "_": "UNDERSCORE",
    "-": "HYPHEN",
    "–": "ENDASH",
    "—": "EMDASH",
    "(": "LRB",
    ")": "RRB",
    "{": "LCB",
    "}": "RCB",
    "[": "LSB",
    "]": "RSB",
    "|": "PIPE",
    "<": "LT",
    ">": "GT",
    "^": "CARET",
    ".": "PERIOD",
    "·": "CDOT",
    "…": "LDOTS",
    ",": "COMMA",
    ":": "COLON",
    "!": "EXCL",
    "?": "QUE",
    ";": "SEMICOLON",
    "%": "PERCENTAGE",
    "$": "DOLLAR",
    "¢": "CENT",
    "€": "EURO",
    "°": "XDEGREE",
    "§": "PARAGRAPH",
    "§§": "PARAGRAPHS",
    "‚": "QUOTE",
    "„": "QUOTE",
    "“": "QUOTE",
    "”": "QUOTE",
    "'": "QUOTE",
    '"': "QUOTE",
    "‘": "QUOTE",
    "`": "QUOTE",
    "´": "QUOTE",
    "ˋ": "QUOTE",
    "«": "QUOTE",
    "»": "QUOTE",
    "²": "SQUARE",
    "³": "CUBE",
    "@": "ATSYMBOL",
    "&": "AMPERSAND",
    "#": "HASHTAG",
    "~": "TILDE",
    "*": "ASTERISK",
    "∞": "INFINITY",
    "=": "EQUALS",
}

MISC_REPLACEMENTS = {
    "m²": "m2",
    "m³": "m3",
    "½": "1SLASH2",
    "™": "TM",
    "©": "COPYRIGHT",
    "Æ": "ae",
    "‼": "EXCLEXCL",
    "⁉": "EXCLQUE",
    "ą": "INVALID",
    "ñ": "INVALID",
    "\u2020": "INVALID",
    "\xe5": "INVALID",
    "\xe7": "INVALID",
    "\xca": "INVALID",
    "\u0131": "INVALID",
    "\u2b07": "INVALID",
    "\u0130": "INVALID",
    "\u2260": "INVALID",
    "\uf0b7": "INVALID",
    "\uf818": "INVALID",
    "\uf0e0": "INVALID",
    "\u00ad": "INVALID",
    "\u200B": "INVALID",
    "\ufe0f": "INVALID",
    "\u010f": "INVALID",
    "\u02c8": "INVALID",
    "\u06de": "INVALID",
    "\u06e9": "INVALID",
    "\u2019": "INVALID",
    "\u2032": "INVALID",
    "\u2501": "INVALID",
    "\u2523": "INVALID",
    "\u252b": "INVALID",
    "\u25ac": "INVALID",
    "\u25cf": "INVALID",
    "\u2605": "INVALID",
    "\u2764": "INVALID",
    "\u2765": "INVALID",
    "\u27a4": "INVALID",
    "\u2935": "INVALID",
    "\u1d00": "INVALID",
    "\u1d05": "INVALID",
    "\u1d07": "INVALID",
    "\u0259": "INVALID",
    "\u0261": "INVALID",
    "\u0262": "INVALID",
    "\u029c": "INVALID",
    "\u1d35": "INVALID",
    "\u026a": "INVALID",
    "\u1d0b": "INVALID",
    "\u1d0d": "INVALID",
    "\u0274": "INVALID",
    "\u014f": "INVALID",
    "\u1d0f": "INVALID",
    "\u0280": "INVALID",
    "\u1d1b": "INVALID",
    "\u028a": "INVALID",
    "\u1d21": "INVALID",
    "\u028f": "INVALID",
    "\u05d0": "INVALID",
    "\u05d5": "INVALID",
    "\u05d7": "INVALID",
    "\u05d9": "INVALID",
    "\u05db": "INVALID",
    "\u05dc": "INVALID",
    "\u05dd": "INVALID",
    "\u05e2": "INVALID",
    "\u05e9": "INVALID",
    "\u200d": "INVALID",
    "\u010d": "INVALID",
    "\u015f": "INVALID",
    "\u017a": "INVALID",
    "\u017b": "INVALID",
    "\u03b1": "INVALID",
    "\u0435": "INVALID",
    "\u043b": "INVALID",
    "\u041c": "INVALID",
    "\u0441": "INVALID",
    "\u516c": "INVALID",
    "\u0650": "INVALID",
    "\u0119": "INVALID",
    "\u0155": "INVALID",
    "\u2010": "INVALID",
    "\u0101": "INVALID",
    "\u0107": "INVALID",
    "\u0282": "INVALID",
    "\u016f": "INVALID",
    "\u0b9c": "INVALID",
    "\u2022": "INVALID",
}
